import moment from "moment-timezone";

import { actionTypes as asyncDataFetchActionTypes } from "@bufferapp/async-data-fetch";
import keyWrapper from "@bufferapp/keywrapper";

export const actionTypes = keyWrapper("DATE_PICKER", {
  OPEN_DATE_PICKER: "OPEN_DATE_PICKER",
  CLOSE_DATE_PICKER: "CLOSE_DATE_PICKER",
  SET_MONTH: "SET_MONTH",
  SET_DATE_RANGE: "SET_DATE_RANGE",
  SET_START_DATE: "SET_START_DATE",
  CLEAR_START_DATE: "CLEAR_START_DATE",
  SET_END_DATE: "SET_END_DATE",
  CLEAR_END_DATE: "CLEAR_END_DATE",
  OPEN_CALENDAR: "OPEN_CALENDAR",
  CLOSE_CALENDAR: "CLOSE_CALENDAR",
});

function getDayFromTimestamp(timestamp) {
  return moment.unix(timestamp).startOf("day");
}

function formatDay(momentDay) {
  return momentDay.format("MM/DD/YYYY");
}

function formatTimestamp(timestamp) {
  const isValidFormat = moment(timestamp, "MM/DD/YYYY").isValid();

  if (isValidFormat) {
    return timestamp;
  }

  return formatDay(getDayFromTimestamp(timestamp));
}

export function calculateDateRange(range) {
  // We need to enfoce the start of the day to make sure
  // that the range is not effected by the time of the day
  let startDate;
  let endDate;

  moment.tz.setDefault('UTC');

  if (typeof range == 'number'){
    startDate = formatDay(moment().startOf('day').subtract(range, 'days'));
    endDate = formatDay(moment().startOf('day').subtract(1, 'days'));
  } else {
      switch (range) {
        case "This month":
          startDate = formatDay(moment().startOf("month"));
          // check to see if it's the first day of the month and if it is, send the same start date
          if (moment().date() === 1) {
            endDate = startDate;
          } else
            endDate = formatDay(moment().startOf("day").subtract(1, "day")); // we gotta check here too
          break;
        case "Last month":
          startDate = formatDay(
            moment().startOf("day").subtract(1, "months").startOf("month")
          );
          endDate = formatDay(
            moment().startOf("day").subtract(1, "months").endOf("month")
          );
          break;
        case "This week":
          startDate = formatDay(moment().startOf("day").startOf("isoWeek"));
          // check to see if it's Monday and if it is, send the same start date
          if (moment().isoWeekday() === 1) {
            endDate = startDate;
          } else
            endDate = formatDay(moment().startOf("day").subtract(1, "day"));
          break;
        case "Last week":
          startDate = formatDay(
            moment().startOf("day").subtract(1, "week").startOf("isoWeek")
          );
          endDate = formatDay(
            moment().startOf("day").subtract(1, "week").endOf("isoWeek")
          );
          break;
      }
  }

  moment.tz.setDefault();
  return { startDate, endDate };
}

export function updatePresets(presets, selectedPreset) {
  return {
    presets: presets.map((preset) => {
      preset.selected = selectedPreset ? selectedPreset.range == preset.range : preset.range === Infinity;
      return preset;
    }),
  };
}

export const presets = [
  {
    name: "This month",
    label: 'This month',
    range: 'This month',
    selected: true,
    disabled: false,
  },
  {
    name: "Last month",
    label: 'Last month',
    range: 'Last month',
    selected: false,
    disabled: false,
  },
  {
    name: "This week",
    label: 'This week',
    range: 'This week',
    selected: false,
    disabled: false,
  },
  {
    name: "Last week",
    label: 'Last week',
    range: 'Last week',
    selected: false,
    disabled: false,
  },
  {
    name: "Custom",
    label: 'Custom',
    range: Infinity,
    selected: false,
    disabled: false,
  },
];

const initialState = {
  loading: false,
  open: false,
  calendarOpen: false,
  minDate: null,
  maxDate: moment.utc().valueOf(),
  month: moment.utc().startOf("month").unix(),
  ...calculateDateRange('This month'),
  presets,
};

export default (state = initialState, action) => {
  switch (action.type) {
    case actionTypes.SET_MONTH:
      return {
        ...state,
        month: action.date,
      };
    case actionTypes.CLEAR_START_DATE:
      return {
        ...state,
        startDate: null,
      };
    case actionTypes.SET_START_DATE:
      return {
        ...state,
        startDate: action.date,
      };
    case actionTypes.CLEAR_END_DATE:
      return {
        ...state,
        endDate: null,
      };
    case actionTypes.SET_END_DATE:
      return {
        ...state,
        endDate: action.date,
      };
    case actionTypes.SET_DATE_RANGE:
      return {
        ...state,
        ...updatePresets(state.presets, action.preset),
        startDate: action.startDate,
        endDate: action.endDate,
        previousStartDate: null,
        previousEndDate: null,
      };
    case `get_report_${asyncDataFetchActionTypes.FETCH_START}`:
      return {
        ...state,
        loading: true,
      };
    case `get_report_${asyncDataFetchActionTypes.FETCH_FAIL}`:
      return {
        ...initialState,
        loading: false,
        hasError: true,
      };
    case `get_report_${asyncDataFetchActionTypes.FETCH_SUCCESS}`:
      return {
        ...state,
        loading: false,
      };
    case actionTypes.OPEN_DATE_PICKER:
      return {
        ...state,
        open: true,
      };
    case actionTypes.CLOSE_DATE_PICKER:
      return {
        ...state,
        open: false,
        calendarOpen: false,
        startDate: state.previousStartDate
          ? state.previousStartDate
          : state.startDate,
        endDate: state.previousEndDate ? state.previousEndDate : state.endDate,
        ...state.previousStartDate ? updatePresets(state.presets, state.previousPreset) : state.presets
      };
    case actionTypes.OPEN_CALENDAR:
      return {
        ...state,
        calendarOpen: true,
        previousStartDate: state.startDate,
        previousEndDate: state.endDate,
        startDate: null,
        endDate: null,
        previousPreset: state.presets.find((preset) => preset.selected),
        ...updatePresets(state.presets, presets[4])
      };
    default:
      return state;
  }
};

export const actions = {
  open: () => ({
    type: actionTypes.OPEN_DATE_PICKER,
  }),
  close: () => ({
    type: actionTypes.CLOSE_DATE_PICKER,
  }),
  openCalendar: () => ({
    type: actionTypes.OPEN_CALENDAR,
  }),
  setStartDate: (date) => ({
    type: actionTypes.SET_START_DATE,
    date: formatTimestamp(date),
  }),
  clearStartDate: () => ({
    type: actionTypes.CLEAR_START_DATE,
  }),
  setEndDate: (date) => ({
    type: actionTypes.SET_END_DATE,
    date: formatTimestamp(date),
  }),
  clearEndDate: () => ({
    type: actionTypes.CLEAR_END_DATE,
  }),
  setDateRange: (startDate, endDate) => ({
    type: actionTypes.SET_DATE_RANGE,
    startDate: formatTimestamp(startDate),
    endDate: formatTimestamp(endDate),
  }),
  setDatePreset: (preset) => ({
    type: actionTypes.SET_DATE_RANGE,
    preset,
    ...calculateDateRange(preset.range),
  }),
  setMonth: (date) => ({
    type: actionTypes.SET_MONTH,
    date,
  }),
  setCurrentTab: (tabId) => ({
    type: actionTypes.SET_CURRENT_TAB,
    tabId,
  }),
  setDateRangeAndPreset: ({ preset, startDate, endDate }) => ({
    type: actionTypes.SET_DATE_RANGE,
    startDate,
    endDate,
    preset,
  }),
};
